//
//  MINTLActionPopupButton.swift
//  ControlSikenjo
//
//  Created by narita on 2018/12/04.
//  Copyright © 2018 narita. All rights reserved.
//

import Cocoa

// super superへのメソッド転送
// http://www.cocoadev.com/index.pl?MovingAroundClassHierarchy
extension NSPopUpButtonCell
{
    @objc func usingSuperclassImplementationTrackMouse(with event: NSEvent,
                                                        in cellFrame: NSRect,
                                                        of controlView: NSView,
                                                        untilMouseUp flag: Bool) -> Bool
    {
        return super.trackMouse(with: event,
                                in: cellFrame,
                                of: controlView,
                                untilMouseUp: flag)
    }
}

class MINTLActionPopupButtonCell: NSPopUpButtonCell
{
    // MARK: - class methods
    
    // MARK: - init methods
    
    override public init(textCell stringValue: String, pullsDown pullDown: Bool)
    {
        super.init(textCell: stringValue, pullsDown: true)

        pullsDown = true
        
        // 最初の1行目が食われてしまう現象をダミーの追加で回避
        insertItem(withTitle: "Dummy Menu", at: 0)
        usesItemFromMenu = false
        
        let theMenuItem = NSMenuItem(title: "", action: nil, keyEquivalent: "")
        theMenuItem.image = NSImage(imageLiteralResourceName: NSImage.actionTemplateName)
        theMenuItem.onStateImage = nil
        theMenuItem.mixedStateImage = nil
        menuItem = theMenuItem

        setButtonType(.pushOnPushOff)

        imagePosition = .imageOnly
        imageScaling = .scaleProportionallyDown
        
        imageDimsWhenDisabled = true
        
        synchronizeTitleAndSelectedItem()
    }
    
    public convenience init(textCell string: String)
    {
        self.init(textCell: string, pullsDown: true)
    }
    
    public convenience init(imageCell image: NSImage?)
    {
        self.init(textCell: "", pullsDown: true)
    }

    public convenience init()
    {
        self.init(textCell: "", pullsDown: true)
    }

    
    // MARK: - dealloc
    
    // MARK: - NSCopying, hash, isEqual:
    
    // MARK: - NSCoding methods

    required init(coder: NSCoder)
    {
        super.init(coder: coder)
        
        pullsDown = true
        
        // 最初の1行目が食われてしまう現象をダミーの追加で回避
        insertItem(withTitle: "Dummy Menu", at: 0)
        usesItemFromMenu = false
        
        let theMenuItem = NSMenuItem(title: "", action: nil, keyEquivalent: "")
        theMenuItem.image = NSImage(imageLiteralResourceName: NSImage.actionTemplateName)
        theMenuItem.onStateImage = nil
        theMenuItem.mixedStateImage = nil
        menuItem = theMenuItem

        setButtonType(.pushOnPushOff)

        imagePosition = .imageOnly
        imageScaling = .scaleProportionallyDown
        
        imageDimsWhenDisabled = true
        
        synchronizeTitleAndSelectedItem()
    }
    
    // MARK: - restorableState methods
    
    // MARK: - life cycle methods
    
    // MARK: - action methods
    
    // MARK: - event handling methods
    
    override func trackMouse(with event: NSEvent,
                             in cellFrame: NSRect,
                             of controlView: NSView,
                             untilMouseUp flag: Bool) -> Bool
    {
        // see http://www.cocoabuilder.com/archive/message/cocoa/2003/10/22/90316
        let theDelayTime = min(0.75,
                               max(0.2,
                                   UserDefaults.standard.double(
                                    forKey: "com.apple.mouse.doubleClickThreshold")))
        
        let theTmpEvent = NSApp.nextEvent(matching: [.leftMouseUp, .leftMouseDragged],
                                          until: Date(timeIntervalSinceNow: theDelayTime),
                                          inMode: .eventTracking,
                                          dequeue: false)

        if theTmpEvent?.type == .leftMouseUp
        {
            return super.usingSuperclassImplementationTrackMouse(with: event,
                                                                   in: cellFrame,
                                                                   of: controlView,
                                                         untilMouseUp: flag)
        }
        else
        {
            return super.trackMouse(with: event,
                                    in: cellFrame,
                                    of: controlView,
                                    untilMouseUp: flag)
        }
    }

    
    
    // MARK: - drawing methods
    
    // MARK: - delegate/datasource methods
    
    // MARK: - accessor methods (in pairs)
    
    override var image: NSImage?
    {
        didSet
        {
            self.menuItem?.image = self.image
        }
    }
    
    // MARK: - Utility methods
}

class MINTLActionPopupButton: NSPopUpButton
{
    // MARK: - class methods
    
    class func cellClass() -> AnyClass?
    {
        return MINTLActionPopupButtonCell.self
    }

    // MARK: - init methods
    
    override init(frame frameRect: NSRect)
    {
        super.init(frame: frameRect)
        self.pullsDown = true
    }
    
    // MARK: - dealloc
    
    // MARK: - NSCopying, hash, isEqual:
    
    // MARK: - NSCoding methods
    
    required init?(coder: NSCoder)
    {
        super.init(coder: coder)
        
        if     ( self.cell == nil )
            ||
               ( self.cell!.className != MINTLActionPopupButtonCell.className() )
        {
            self.cell = MINTLActionPopupButtonCell(textCell: "", pullsDown: true)
        }
    }
    
    // MARK: - restorableState methods
    
    // MARK: - life cycle methods
    
    // MARK: - action methods
    
    // MARK: - event handling methods
    
    // MARK: - drawing methods

    // MARK: - delegate/datasource methods
    
    // MARK: - accessor methods (in pairs)
    
    // MARK: - Utility methods
}
